package nexus.billing

import android.app.Activity
import android.content.Context
import androidx.appcompat.app.AppCompatActivity
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.catch
import kotlinx.coroutines.flow.flow
import kotlinx.coroutines.launch
import kotlinx.coroutines.suspendCancellableCoroutine
import nexus.appui.ui.BaseFragment
import nexus.billing.methods.common.Descriptions
import nexus.billing.methods.common.Provider
import nexus.billing.methods.common.Route
import nexus.billing.methods.common.Units
import nexus.billing.methods.notifications.ProfilePlanUpdateNotify
import nexus.billing.methods.notifications.ProfileUpdateNotify
import nexus.billing.methods.plan.Plan
import nexus.billing.methods.profile.ProfilePlan
import nexus.billing.methods.profile.ProfileService
import nexus.billing.methods.requests.*
import nexus.billing.methods.requests.plan.GetAllPlans
import nexus.billing.methods.requests.plan.GetPlan
import nexus.billing.methods.requests.plan.PlanSelect
import nexus.billing.methods.requests.plan.PlanServiceRoute
import nexus.billing.methods.requests.profileplan.GetProfilePlan
import nexus.billing.methods.requests.profileplan.ProfilePlanActivate
import nexus.billing.methods.requests.profileplan.ProfilePlanAssetSelect
import nexus.billing.methods.requests.profileplan.ProfilePlanAssets
import nexus.billing.methods.requests.profileplan.ProfilePlanRemove
import nexus.billing.methods.requests.profileplan.ProfilePlanRoute
import nexus.billing.methods.requests.profileplan.ProfilePlanSync
import nexus.billing.methods.requests.profileplan.external.GetProfilePlanProviders
import nexus.billing.methods.requests.profileplan.external.ProfilePlanRegister
import nexus.billing.methods.requests.profileplan.external.ProfilePlanValidate
import nexus.billing.methods.requests.profileplan.service.ProfilePlanServiceActivate
import nexus.billing.methods.requests.profileplan.service.ProfilePlanServiceAdd
import nexus.billing.methods.requests.profileplan.service.ProfilePlanServiceForwarding
import nexus.billing.methods.requests.profileplan.service.ProfilePlanServiceRemove
import nexus.billing.methods.requests.profileplan.service.ProfilePlanServiceSMS
import nexus.billing.methods.requests.profileplan.service.ProfilePlanServiceVoiceMailSet
import nexus.billing.methods.requests.voicemail.AddVoiceMail
import nexus.billing.methods.requests.voicemail.GetVoiceMail
import nexus.billing.methods.requests.voicemail.RemoveVoiceMail
import nexus.billing.methods.service.Service
import nexus.billing.models.DescriptionObject
import nexus.billing.models.PlanInfo
import nexus.billing.models.PlanObject
import nexus.billing.models.planDesc
import nexus.billing.ui.profile.SelectPlanDialog
import nexus.core.NCPCore
import nexus.core.controllers.NetworkController
import nexus.core.controllers.file
import nexus.core.kit.Account
import nexus.core.kit.AnyController
import nexus.core.util.Logger
import nexus.core.util.NotifyObject
import nexus.core.util.NotifyType
import java.io.File
import kotlin.coroutines.resume
import kotlin.coroutines.resumeWithException

interface BillingDelegate {
    fun openSupportChat(activity: AppCompatActivity, containerId: Int)
    fun openTopUpBalance(activity: Activity, profilePaymentSource: String)
    suspend fun checkBalance(amount: Int): Boolean
}

interface BillingPlansDelegate {
    fun selectPlan(fragment: BaseFragment, type: ProfilePlanRoute.RouteType, phoneNumber: String, callback: (profilePlanId: String, title: String?, icon: String?, number: String?) -> Unit)
    fun getPlanInfo(context: Context, profilePlanId: String, callback: (title: String?, icon: String?, number: String?) -> Unit)
    suspend fun planRoutes(type: ProfilePlanRoute.RouteType, destination: String, profilePlanId: String?): Array<ProfilePlanRoute.Response.Routes>
    suspend fun sendSms(profilePlanId: String, profileServiceId: String, phoneNumber: String, text: String): String?
}

class BillingController(core: NCPCore): AnyController(core) {

    val TAG = "BillingController"
    private val descUpdateTime = 604800000 // 7 days
    init {
        core.network.subscribe(ProfileUpdateNotify(), ::profileUpdated)
        core.network.subscribe(ProfilePlanUpdateNotify(), ::profilePlanUpdated)
    }

    var delegate: BillingDelegate? = null
    var planDelegate: BillingPlansDelegate = object : BillingPlansDelegate {
        override fun selectPlan(fragment: BaseFragment, type: ProfilePlanRoute.RouteType, phoneNumber: String, callback: (profilePlanId: String, title: String?, icon: String?, number: String?) -> Unit) {
            SelectPlanDialog.getInstance(fragment, type, phoneNumber).show(object : SelectPlanDialog.SelectPlanListener {
                override fun planSelected(planObject: PlanObject) {
                    val number = planObject.plan.services.find { it.asset != null }?.asset?.number
                    callback.invoke(planObject.id, planObject.planDesc?.title ?: planObject.planInfo.name, planObject.planDesc?.logo, number)
                }
            })
        }

        override fun getPlanInfo(context: Context, profilePlanId: String, callback: (title: String?, icon: String?, number: String?) -> Unit) {
            CoroutineScope(Dispatchers.IO).launch {
                getProfilePlans(NCPCore.currentLanguage).find { it.id == profilePlanId }?.let { planObject ->
                    val number = planObject.plan.services.find { it.asset != null }?.asset?.number
                    callback.invoke(planObject.planDesc?.title ?: planObject.planInfo.name, planObject.planDesc?.logo, number)
                }
            }
        }

        override suspend fun planRoutes(type: ProfilePlanRoute.RouteType, destination: String, profilePlanId: String?): Array<ProfilePlanRoute.Response.Routes> {
            return this@BillingController.requestProfilePlanRoute(type, destination, profilePlanId)?.routes ?: emptyArray()
        }

        override suspend fun sendSms(profilePlanId: String, profileServiceId: String, phoneNumber: String, text: String): String? {
            return this@BillingController.requestServiceSMS(profilePlanId, profileServiceId, phoneNumber, text)?.id
        }
    }

    override fun switchAccount(account: Account?) {}

    override fun socketChange(state: NetworkController.ConnectionState) {}

    override fun reSync() {
        CoroutineScope(Dispatchers.IO).launch {
            this@BillingController.core.database.run {
                PlanObject.deleteAll(this)
                DescriptionObject.deleteAll(this)
            }
            this@BillingController.getProfilePlans(NCPCore.currentLanguage)
        }
    }

    private fun profileUpdated(params: ProfileUpdateNotify.Params) {
        CoroutineScope(Dispatchers.IO).launch {
            for (plan in params.plans) {
                saveProfilePlan(NCPCore.currentLanguage, plan, true)
            }
        }
    }

    private fun profilePlanUpdated(params: ProfilePlanUpdateNotify.Params) {
        CoroutineScope(Dispatchers.IO).launch {
            if (params.state == ProfilePlan.State.removed) {
                this@BillingController.core.database.run {
                    PlanObject.findBy(this, params.id)?.let {
                        it.delete(this)
                        this@BillingController.core.billingKit.planChangeEvent.accept(NotifyObject(NotifyType.Delete, it))
                    }
                }
            } else {
                saveProfilePlan(NCPCore.currentLanguage, params, true)
            }
        }
    }

    private suspend fun saveProfilePlan(language: String, profilePlan: ProfilePlan, notify: Boolean): PlanObject? {
        try {
            val getPlan = this.core.network.request(GetPlan(GetPlan.Params(profilePlan.planID)), NetworkController.RequestType.SOCKET)
            getPlan.result?.let { publicPlan ->
                val info = PlanInfo(publicPlan.id, publicPlan.name, publicPlan.provider, publicPlan.category, publicPlan.verificationRequired, publicPlan.type, publicPlan.services)
                val serviceIDs = ArrayList(profilePlan.services.map { it.serviceID })
                serviceIDs.addAll(publicPlan.services.map { it.id })
                val descriptions = this.getDescriptions(profilePlan.planID, info.category.id, arrayOf(info.provider.id), serviceIDs.distinct().toTypedArray(), language)
                if (descriptions.isNotEmpty()) {
                    info.descriptions = descriptions
                    var savedPlan: PlanObject? = null
                    var update = true
                    this.core.database.run {
                        savedPlan = PlanObject.findBy(this, profilePlan.id)?.let {
                            it.plan = profilePlan
                            it.planInfo = info
                            it.update(this)
                            it
                        } ?: run {
                            val newPlan = PlanObject(profilePlan.id, language, profilePlan, info)
                            newPlan.insert(this)
                            update = false
                            newPlan
                        }
                    }
                    if (notify) {
                        savedPlan?.let {
                            this.core.billingKit.planChangeEvent.accept(NotifyObject(if (update) NotifyType.Update else NotifyType.Create, it))
                        }
                    }
                    return savedPlan
                } else {
                    Logger.log("saveProfilePlan", "Descriptions not found")
                    return null
                }
            } ?: run {
                return null
            }
        } catch (ex: Exception) {
            ex.printStackTrace()
            return null
        }
    }

    suspend fun getProfilePlans(language: String): ArrayList<PlanObject> {
        val list = ArrayList<PlanObject>()
        this.core.database.run {
            list.addAll(PlanObject.findAll(this, language))
        }
        if (list.isEmpty()) {
            val response = this.core.network.request(GetProfile(GetProfile.Params()), NetworkController.RequestType.SOCKET)
            response.result?.let { profile ->
                for (profilePlan in profile.plans) {
                    saveProfilePlan(language, profilePlan, false)?.let { planObject ->
                        list.add(planObject)
                    }
                }
            }
            response.error?.let { err ->
                Logger.log("getProfilePlans", err)
            }
        } else {
            list.forEach { it.fillDescriptions(language) }
        }
        return list
    }

    fun updateProfilePlans(language: String, force: Boolean = false) {
        CoroutineScope(Dispatchers.IO).launch {
            val getProfile = core.network.request(GetProfile(GetProfile.Params()), NetworkController.RequestType.SOCKET)
            getProfile.result?.let { profile ->
                try {
                    val forUpdate = ArrayList<ProfilePlan>()
                    this@BillingController.core.database.run {
                        if (force) {
                            forUpdate.addAll(profile.plans)
                            PlanObject.deleteAll(this)
                        } else {
                            for (plan in profile.plans) {
                                val dbPlan = PlanObject.findBy(this, plan.id)
                                if (dbPlan == null) {
                                    forUpdate.add(plan)
                                }
                            }
                        }
                    }
                    for (profilePlan in forUpdate) {
                        saveProfilePlan(language, profilePlan, true)
                    }
                } catch (e: Exception) {
                    Logger.log("updateProfilePlans", e)
                }
            }
            getProfile.error?.let {
                Logger.log("updateProfilePlans", it)
            }
        }
    }

    @Throws(Exception::class)
    suspend fun requestAllPlans(language: String): ArrayList<PlanInfo> {
        val plans = ArrayList<PlanInfo>()
        val response = this.core.network.request(GetAllPlans(GetAllPlans.Params()), NetworkController.RequestType.SOCKET)
        response.result?.let { resp ->
            for (plan in resp.plans) {
                val serviceIds = plan.services.map { it.id }.toTypedArray()
                val info = PlanInfo(plan.id, plan.name, plan.provider, plan.category, plan.verificationRequired, plan.type, plan.services)
                info.descriptions = this.getDescriptions(plan.id, plan.category.id, arrayOf(plan.provider.id), serviceIds, language)
                plans.add(info)
            }
        }
        response.error?.let {
            Logger.log(it)
        }
        return plans
    }

    suspend fun requestGetPlan(planId: String): Plan? = suspendCancellableCoroutine { obs ->
        val req = GetPlan(GetPlan.Params(planId))
        this.core.network.request(req, NetworkController.RequestType.SOCKET, { response ->
            response.result?.let { publicPlan ->
                obs.resume(publicPlan)
            }
            response.error?.let { err ->
                Logger.log("requestGetPlan", err)
                obs.resume(null)
            }
        })
    }

    suspend fun requestGetPlanWithInfo(planId: String): PlanInfo? {
        var planInfo: PlanInfo? = null
        val response = this.core.network.request(GetPlan(GetPlan.Params(planId)), NetworkController.RequestType.SOCKET)
        response.result?.let { publicPlan ->
            planInfo = PlanInfo(publicPlan.id, publicPlan.name, publicPlan.provider, publicPlan.category, publicPlan.verificationRequired, publicPlan.type, publicPlan.services)
            planInfo?.descriptions = getDescriptions(publicPlan.id, publicPlan.category.id, arrayOf(publicPlan.provider.id), publicPlan.services.map { it.id }.toTypedArray(), NCPCore.currentLanguage)
        }
        response.error?.let { err ->
            Logger.log("requestGetPlan", err)
        }
        return planInfo
    }

    suspend fun requestGetDescriptions(
        language: String,
        planIDs: Array<String>? = null,
        serviceIDs: Array<String>? = null,
        providerIDs: Array<String>? = null,
        planCategoryIDs: Array<String>? = null
    ): Array<Descriptions> = suspendCancellableCoroutine { obs ->
        val req = GetDescriptions(GetDescriptions.Params(language, planIDs, serviceIDs, providerIDs, planCategoryIDs))
        this.core.network.request(req, NetworkController.RequestType.SOCKET, { response ->
            response.result?.let { resp ->
                obs.resume(resp.descriptions)
            }
            response.error?.let { err ->
                Logger.log("requestGetDescriptions", err)
                obs.resume(emptyArray())
            }
        })
    }

    suspend fun requestGetProfilePlan(profilePlanId: String): PlanObject? {
        var plan: PlanObject? = null
        val response = this.core.network.request(GetProfilePlan(GetProfilePlan.Params(profilePlanId)), NetworkController.RequestType.SOCKET)
        response.result?.let { resp ->
            try {
                plan = saveProfilePlan(NCPCore.currentLanguage, resp, false)
            } catch (ex: Exception) {
                ex.printStackTrace()
            }
        }
        response.error?.let { err ->
            Logger.log("requestGetProfilePlan", err)
        }
        return plan
    }

    suspend fun getProfilePlan(profilePlanId: String): PlanObject? {
        var plan: PlanObject? = null
        this.core.database.run {
            plan = PlanObject.findBy(this, profilePlanId)
        }
        plan?.fillDescriptions(NCPCore.currentLanguage, true) ?: run {
            plan = this.requestGetProfilePlan(profilePlanId)
        }
        return plan
    }

    private suspend fun PlanObject.fillDescriptions(language: String, publicServiceDesc: Boolean = false) {
        val serviceIds = ArrayList(this.plan.services.mapNotNull { service ->
            if (service.state != ProfileService.State.removed) service.serviceID else null
        })
        if (publicServiceDesc) {
            serviceIds.addAll(this.planInfo.services.map { it.id })
        }
        this.planInfo.descriptions = this@BillingController.getDescriptions(this.planInfo.planID, this.planInfo.category.id, arrayOf(this.planInfo.provider.id), serviceIds.toTypedArray(), language)
    }

    suspend fun requestPlanAssetsSelect(profilePlanId: String, serviceId: String, assetId: String): PlanObject? {
        val req = ProfilePlanAssetSelect(ProfilePlanAssetSelect.Params(profilePlanId, serviceId, assetId))
        val response = this.core.network.request(req, NetworkController.RequestType.SOCKET)
        var plan: PlanObject? = null
        response.result?.let { resp ->
            plan = saveProfilePlan(NCPCore.currentLanguage, resp, true)
        }
        response.error?.let { err ->
            Logger.log("requestPlanAssetsSelect", err)
        }
        return plan
    }

    @Throws(Exception::class)
    suspend fun requestPlanAssets(profilePlanId: String): List<Pair<Service, DescriptionObject?>> {
        val req = ProfilePlanAssets(ProfilePlanAssets.Params(profilePlanId))
        val assets = this.core.network.request(req, NetworkController.RequestType.SOCKET)
        return assets.result?.let { result ->
            val descriptions = this.getDescriptions(null, null, null, result.assets.map { it.id }.toTypedArray(), NCPCore.currentLanguage)
            result.assets.map { service ->
                val serviceDesc = descriptions.find { it.serviceId == service.id }
                Pair(service, serviceDesc)
            }
        } ?: run {
            emptyList()
        }
    }

    suspend fun requestPlanServiceRoutes(planId: String, serviceId: String): Array<Route> = suspendCancellableCoroutine { obs ->
        val req = PlanServiceRoute(PlanServiceRoute.Params(planId, serviceId))
        this.core.network.request(req, NetworkController.RequestType.SOCKET, { response ->
            response.result?.let { resp ->
                obs.resume(resp.routes)
            }
            response.error?.let { err ->
                Logger.log("requestPlanServiceRoutes", err)
                obs.resume(emptyArray())
            }
        })
    }

    suspend fun requestSelectPlan(planId: String): PlanObject? {
        val req = PlanSelect(PlanSelect.Params(planId))
        val response = this.core.network.request(req, NetworkController.RequestType.SOCKET)
        var plan: PlanObject? = null
        response.result?.let { resp ->
            plan = saveProfilePlan(NCPCore.currentLanguage, resp, true)
        }
        response.error?.let { err ->
            Logger.log("requestSelectPlan", err)
        }
        return plan
    }

    suspend fun requestPlanRemove(profilePlanId: String): Boolean {
        val req = ProfilePlanRemove(ProfilePlanRemove.Params(profilePlanId))
        val response = this.core.network.request(req, NetworkController.RequestType.SOCKET)
        var result = false
        response.result?.let { resp ->
            this.core.database.run {
                val deleted = PlanObject.findBy(this, profilePlanId)
                deleted?.let {
                    it.delete(this)
                    this@BillingController.core.billingKit.planChangeEvent.accept(NotifyObject(NotifyType.Delete, deleted))
                }
                result = true
            }
        }
        response.error?.let { err ->
            Logger.log("requestPlanRemove", err)
            result = false
        }
        return result
    }

    suspend fun requestPlanActivate(profilePlanId: String): PlanObject? {
        val req = ProfilePlanActivate(ProfilePlanActivate.Params(profilePlanId))
        val response = this.core.network.request(req, NetworkController.RequestType.SOCKET)
        val plan = response.result?.let { resp ->
            saveProfilePlan(NCPCore.currentLanguage, resp, true)
        }
        response.error?.let { err ->
            Logger.log("requestPlanActivate", err)
        }
        return plan
    }

    suspend fun requestPlanServiceActivate(profilePlanId: String, profilePlanServiceId: String): PlanObject? {
        val req = ProfilePlanServiceActivate(ProfilePlanServiceActivate.Params(profilePlanId, profilePlanServiceId))
        val response = this.core.network.request(req, NetworkController.RequestType.SOCKET)
        val plan = response.result?.let { resp ->
            saveProfilePlan(NCPCore.currentLanguage, resp, true)
        }
        response.error?.let { err ->
            Logger.log("requestPlanServiceActivate", err)
        }
        return plan
    }

    suspend fun requestPlanServiceAdd(profilePlanId: String, profilePlanServiceId: String): PlanObject? {
        val req = ProfilePlanServiceAdd(ProfilePlanServiceAdd.Params(profilePlanId, profilePlanServiceId))
        val response = this.core.network.request(req, NetworkController.RequestType.SOCKET)
        val plan = response.result?.let { resp ->
            saveProfilePlan(NCPCore.currentLanguage, resp, true)
        }
        response.error?.let { err ->
            Logger.log("requestPlanServiceAdd", err)
        }
        return plan
    }

    suspend fun requestPlanServiceRemove(profilePlanId: String, profilePlanServiceId: String): PlanObject? {
        val req = ProfilePlanServiceRemove(ProfilePlanServiceRemove.Params(profilePlanId, profilePlanServiceId))
        val response = this.core.network.request(req, NetworkController.RequestType.SOCKET)
        val plan = response.result?.let { resp ->
            saveProfilePlan(NCPCore.currentLanguage, resp, true)
        }
        response.error?.let { err ->
            Logger.log("requestPlanServiceAdd", err)
        }
        return plan
    }

    suspend fun requestProfilePlanRoute(type: ProfilePlanRoute.RouteType, destination: String, profilePlanId: String?): ProfilePlanRoute.Response? {
        val req = ProfilePlanRoute(ProfilePlanRoute.Params(type, destination, profilePlanId))
        val response = this.core.network.request(req, NetworkController.RequestType.SOCKET)
        var result: ProfilePlanRoute.Response? = null
        response.result?.let { resp ->
            this.core.database.run {
                for (route in resp.routes.filter { it.`package` != null }) {
                    route.`package`?.profilePlanID?.let { profilePlanId ->
                        PlanObject.findBy(this, profilePlanId)?.let { plan ->
                            val index = plan.plan.services.indexOfFirst { it.id == route.`package`?.profilePlanServiceID }
                            if (index > -1) {
                                val service = plan.plan.services[index]
                                if (service.units?.available != route.`package`?.available || service.units?.total != route.`package`?.total) {
                                    service.units = Units(route.`package`?.available ?: 0, route.`package`?.total ?: 0)
                                    plan.plan.services[index] = service
                                    plan.update(this)
                                }
                            }
                        }
                    }
                }
            }
            result = resp
        }
        response.error?.let { err ->
            Logger.log("requestProfilePlanRoute", err)
        }
        return result
    }

    suspend fun requestPlanSync(): ProfilePlanSync.Response? = suspendCancellableCoroutine { obs ->
        val req = ProfilePlanSync(ProfilePlanSync.Params())
        this.core.network.request(req, NetworkController.RequestType.SOCKET, { response ->
            response.result?.let { resp ->
                obs.resume(resp)
            }
            response.error?.let { err ->
                obs.resume(null)
            }
        })
    }

    suspend fun requestServiceSMS(profilePlanId: String, profilePlanServiceId: String, destination: String, body: String): ProfilePlanServiceSMS.Response? = suspendCancellableCoroutine { obs ->
        val req = ProfilePlanServiceSMS(ProfilePlanServiceSMS.Params(profilePlanId, profilePlanServiceId, destination, body))
        this.core.network.request(req, NetworkController.RequestType.SOCKET, { response ->
            response.result?.let { resp ->
                obs.resume(resp)
            }
            response.error?.let { err ->
                obs.resume(null)
            }
        })
    }

    suspend fun requestServiceForwarding(profilePlanId: String, profilePlanServiceId: String, forwarding: Boolean): ProfilePlanServiceForwarding.Response? = suspendCancellableCoroutine { obs ->
        val req = ProfilePlanServiceForwarding(ProfilePlanServiceForwarding.Params(profilePlanId, profilePlanServiceId, forwarding))
        this.core.network.request(req, NetworkController.RequestType.SOCKET, { response ->
            response.result?.let { resp ->
                this.core.database.run {
                    PlanObject.findBy(this, profilePlanId)?.let {
                        val index = it.plan.services.indexOfFirst { it.id == profilePlanServiceId }
                        if (index != -1) {
                            it.plan.services[index] = resp
                        }
                        it.update(this)
                        this@BillingController.core.billingKit.planChangeEvent.accept(NotifyObject(NotifyType.Update, it))
                    }
                }
                obs.resume(resp)
            }
            response.error?.let { err ->
                obs.resume(null)
            }
        })
    }

    suspend fun requestSetVoiceMail(profilePlanId: String, profilePlanServiceId: String, id: String): ProfilePlanServiceVoiceMailSet.Response? = suspendCancellableCoroutine { obs ->
        val req = ProfilePlanServiceVoiceMailSet(ProfilePlanServiceVoiceMailSet.Params(profilePlanId, profilePlanServiceId, id))
        this.core.network.request(req, NetworkController.RequestType.SOCKET, { response ->
            response.result?.let { resp ->
                this.core.database.run {
                    PlanObject.findBy(this, profilePlanId)?.let {
                        val index = it.plan.services.indexOfFirst { it.id == profilePlanServiceId }
                        if (index != -1) {
                            it.plan.services[index] = resp
                        }
                        it.update(this)
                        this@BillingController.core.billingKit.planChangeEvent.accept(NotifyObject(NotifyType.Update, it))
                    }
                }
                obs.resume(resp)
            }
            response.error?.let { err ->
                obs.resume(null)
            }
        })
    }

    suspend fun requestAddVoiceMail(fileId: String, name: String): AddVoiceMail.Response? = suspendCancellableCoroutine { obs ->
        val req = AddVoiceMail(AddVoiceMail.Params(fileId, name))
        this.core.network.request(req, NetworkController.RequestType.SOCKET, { response ->
            response.result?.let { resp ->
                obs.resume(resp)
            }
            response.error?.let { err ->
                obs.resume(null)
            }
        })
    }

    suspend fun requestRemoveVoiceMail(id: String): RemoveVoiceMail.Response? = suspendCancellableCoroutine { obs ->
        val req = RemoveVoiceMail(RemoveVoiceMail.Params(id))
        this.core.network.request(req, NetworkController.RequestType.SOCKET, { response ->
            response.result?.let { resp ->
                obs.resume(resp)
            }
            response.error?.let { err ->
                obs.resume(null)
            }
        })
    }

    suspend fun requestGetVoiceMail(): GetVoiceMail.Response? = suspendCancellableCoroutine { obs ->
        val req = GetVoiceMail(GetVoiceMail.Params())
        this.core.network.request(req, NetworkController.RequestType.SOCKET, timeOut = 15, completion = { response ->
            response.result?.let { resp ->
                obs.resume(resp)
            }
            response.error?.let { err ->
                obs.resume(null)
            }
        })
    }

    suspend fun requestProfilePlanRegister(providerId: String, login: ProfilePlanRegister.Login?, validate: ProfilePlanRegister.Validate?): PlanObject? {
        val req = ProfilePlanRegister(ProfilePlanRegister.Params(providerId, ProfilePlanRegister.Credentials(login, validate)))
        val response = this.core.network.request(req, NetworkController.RequestType.SOCKET)
        var plan: PlanObject? = null
        response.result?.let { resp ->
            plan = saveProfilePlan(NCPCore.currentLanguage, resp, true)
        }
        response.error?.let { err ->
            Logger.log("requestProfilePlanRegister", err)
        }
        return plan
    }

    suspend fun requestProfilePlanValidate(providerId: String, login: ProfilePlanValidate.Login?, phone: String?): ProfilePlanValidate.Response? = suspendCancellableCoroutine { obs ->
        val req = ProfilePlanValidate(ProfilePlanValidate.Params(providerId, ProfilePlanValidate.Credentials(login, if (phone != null) ProfilePlanValidate.Phone(phone) else null)))
        this.core.network.request(req, NetworkController.RequestType.SOCKET, { response ->
            response.result?.let { resp ->
                obs.resume(resp)
            }
            response.error?.let { err ->
                Logger.log("requestProfilePlanValidate", err)
                obs.resumeWithException(Exception(err.message))
            }
        }, 60)
    }

    suspend fun requestProfilePlanProviders(): List<Pair<Provider, DescriptionObject>> {
        val req = GetProfilePlanProviders(GetProfilePlanProviders.Params())
        val providers = this.core.network.request(req, NetworkController.RequestType.SOCKET).result?.providers ?: emptyArray()
        val descriptions = this.getDescriptions(null, null, providers.map { it.id }.toTypedArray(), null, NCPCore.currentLanguage)
        return providers.mapNotNull { provider ->
            val desc = descriptions.find { it.providerId == provider.id }
            if (desc != null) {
                Pair(provider, desc)
            } else {
                null
            }
        }
    }

    suspend fun getDescriptions(planId: String?, categoryId: String?, providerIds: Array<String>?, serviceIds: Array<String>?, language: String): ArrayList<DescriptionObject> {
        val descriptions = ArrayList<DescriptionObject>()
        val getDesc = GetDescriptions.Params(language)
        this.core.database.run {
            try {
                planId?.let { plan ->
                    val find = DescriptionObject.findByPlan(this, plan, language)
                    if (find == null || (System.currentTimeMillis() - find.updatedAt) >= this@BillingController.descUpdateTime) {
                        getDesc.planIDs = arrayOf(plan)
                    } else {
                        descriptions.add(find)
                    }
                }
                categoryId?.let { category ->
                    val find = DescriptionObject.findByCategory(this, category, language)
                    if (find == null || (System.currentTimeMillis() - find.updatedAt) >= this@BillingController.descUpdateTime) {
                        getDesc.planCategoryIDs = arrayOf(category)
                    } else {
                        descriptions.add(find)
                    }
                }
                providerIds?.let { provider ->
                    val find = DescriptionObject.findByProviders(this, provider, language)
                        .filter { (System.currentTimeMillis() - it.updatedAt) < this@BillingController.descUpdateTime }
                    descriptions.addAll(find)
                    if (find.size < providerIds.size) {
                        getDesc.providerIDs = providerIds.filter { id -> !find.any { it.id == id } }.toTypedArray()
                    }
                }
                serviceIds?.let { services ->
                    val find = DescriptionObject.findByServices(this, services, language)
                        .filter { (System.currentTimeMillis() - it.updatedAt) < this@BillingController.descUpdateTime }
                    descriptions.addAll(find)
                    if (find.size < serviceIds.size) {
                        getDesc.serviceIDs = serviceIds.filter { id -> !find.any { it.id == id } }.toTypedArray()
                    }
                }
            } catch(ex: Exception) {
                ex.printStackTrace()
            }
        }
        if (getDesc.planIDs != null || getDesc.planCategoryIDs != null || getDesc.providerIDs != null || getDesc.serviceIDs != null) {
            val req = this.core.network.request(GetDescriptions(getDesc), NetworkController.RequestType.SOCKET)
            req.result?.descriptions?.let { forSave ->
                this.core.database.run {
                    try {
                        for (d in forSave) {
                            var isNew = false
                            var desc = DescriptionObject.findBy(this, d.id)
                            if (desc == null) {
                                desc = DescriptionObject(d.id, d.language)
                                isNew = true
                            }
                            desc.apply {
                                this.providerId = d.providerID
                                this.planId = d.planID
                                this.planCategoryId = d.planCategoryID
                                this.serviceId = d.serviceID
                                this.name = d.name
                                this.title = d.title
                                this.shortText = d.shortText
                                this.longText = d.longText
                                this.backgroundImage = d.backgroundImage
                                this.logo = d.logo
                                this.icon = d.icon
                                this.updatedAt = System.currentTimeMillis().toDouble()
                            }
                            if (isNew) desc.insert(this) else desc.update(this)
                            descriptions.add(desc)
                        }
                    } catch(ex: Exception) {
                        ex.printStackTrace()
                    }
                }
            }
        }
        return descriptions
    }

    suspend fun loadImage(fileId: String, thumb: Boolean): Flow<File> {
        return flow {
            try {
                NCPCore.instance.file.loadImage(fileId, thumb).catch { error ->
                    error.printStackTrace()
                }.collect {
                    emit(it)
                }
            } catch (ex: Exception) {
                Logger.log("BillingController:loadImage", ex.toString())
            }
        }
    }
}