package nexus.billing.models

import android.annotation.SuppressLint
import android.content.ContentValues
import android.database.Cursor
import com.bloomberg.selekt.android.ConflictAlgorithm
import com.bloomberg.selekt.android.SQLiteDatabase
import nexus.billing.methods.profile.ProfilePlan
import nexus.billing.methods.profile.ProfileService
import nexus.core.db.DAO
import nexus.core.db.DBObject
import nexus.core.jsonSerializer
import nexus.core.util.Currency

class PlanObject(
    var id: String,
    var language: String,
    var plan: ProfilePlan,
    var planInfo: PlanInfo
) : DBObject {

    val price: Int
        get() {
            var sum = 0
            this.plan.services.filter { it.state != ProfileService.State.removed && it.activation == ProfileService.Activation.auto }.forEach { service ->
                sum += (this.planInfo.services.find { it.id == service.serviceID }?.recurringAmount ?: 0)
            }
            return sum
        }

    val currency: Currency
        get() = this.planInfo.services.firstOrNull()?.currency ?: Currency.USD

    fun findServicePrice(serviceId: String): Int? {
        return this.planInfo.services.find { it.id == serviceId }?.recurringAmount
    }

    companion object : DAO<PlanObject, String> {
        override var table: String = "plan_table"

        const val ID = "id"
        const val LANGUAGE = "language"
        const val PLAN = "plan"
        const val PLAN_INFO = "planInfo"

        override fun createTable(): String {
            return "CREATE TABLE IF NOT EXISTS $table ('$ID' TEXT PRIMARY KEY NOT NULL, '$LANGUAGE' TEXT NOT NULL, '$PLAN' TEXT, '$PLAN_INFO' TEXT)"
        }

        override fun findAll(db: SQLiteDatabase): ArrayList<PlanObject> {
            val cursor = db.query("SELECT * FROM $table", null)
            val items = ArrayList<PlanObject>()
            while (cursor.moveToNext()) {
                items.add(this.unBox(db, cursor))
            }
            return items
        }

        fun findAll(db: SQLiteDatabase, language: String): ArrayList<PlanObject> {
            val cursor = db.query("SELECT * FROM $table where $LANGUAGE = '$language'", null)
            val items = ArrayList<PlanObject>()
            while (cursor.moveToNext()) {
                items.add(this.unBox(db, cursor))
            }
            return items
        }

        override fun findBy(db: SQLiteDatabase, id: String): PlanObject? {
            val cursor: Cursor = db.query("SELECT * FROM $table WHERE $ID = '$id'", null)
            if (cursor.moveToFirst()) {
                return this.unBox(db, cursor)
            }
            return null
        }

        fun deleteAll(db: SQLiteDatabase): Int {
            return db.delete(table, "", null)
        }

        @SuppressLint("Range")
        override fun unBox(db: SQLiteDatabase, cursor: Cursor): PlanObject {
            val id = cursor.getString(cursor.getColumnIndex(ID))
            val language = cursor.getString(cursor.getColumnIndex(LANGUAGE))
            val plan = cursor.getString(cursor.getColumnIndex(PLAN)).let { jsonSerializer.fromJson(it, ProfilePlan::class.java) }
            val publicPlan = cursor.getString(cursor.getColumnIndex(PLAN_INFO)).let { jsonSerializer.fromJson(it, PlanInfo::class.java) }
            return PlanObject(id, language, plan, publicPlan)
        }
    }

    override fun insert(db: SQLiteDatabase): Long {
        val values = ContentValues()
        values.apply {
            put(ID, this@PlanObject.id)
            put(LANGUAGE, this@PlanObject.language)
            put(PLAN, this@PlanObject.plan.let { jsonSerializer.toJson(it) })
            put(PLAN_INFO, this@PlanObject.planInfo.let { jsonSerializer.toJson(it) })
        }
        return db.insert(table, values, ConflictAlgorithm.REPLACE)
    }

    override fun update(db: SQLiteDatabase): Int {
        val values = ContentValues()
        values.apply {
            put(LANGUAGE, this@PlanObject.language)
            put(PLAN, this@PlanObject.plan.let { jsonSerializer.toJson(it) })
            put(PLAN_INFO, this@PlanObject.planInfo.let { jsonSerializer.toJson(it) })
        }
        return db.update(table, values, "$ID = '${this.id}'", null, ConflictAlgorithm.REPLACE)
    }

    override fun delete(db: SQLiteDatabase): Int {
        return db.delete(table, "$ID = '${this.id}'", null)
    }
}