package nexus.billing.ui

import android.app.Activity
import android.graphics.Color
import android.graphics.Paint
import android.graphics.drawable.ClipDrawable
import android.graphics.drawable.Drawable
import android.graphics.drawable.LayerDrawable
import android.graphics.drawable.ShapeDrawable
import android.graphics.drawable.StateListDrawable
import android.graphics.drawable.shapes.RoundRectShape
import android.net.Uri
import android.view.Gravity
import android.view.View
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.Toast
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.flow.catch
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext
import nexus.appui.Theme
import nexus.appui.Theme.dp
import nexus.appui.utils.ViewUtils
import nexus.billing.billingKit
import nexus.core.NCPCore
import nexus.core.util.Currency
import nexus.core.util.Logger


fun loadImage(fileId: String, img: ImageView, errorImg: Int? = null) {
    CoroutineScope(Dispatchers.IO).launch {
        NCPCore.instance.billingKit.controller.loadImage(fileId, true).catch { error ->
            Logger.log("Utils:loadImage", error.toString())
            errorImg?.let {icon ->
                withContext(Dispatchers.Main) {
                    img.setImageResource(icon)
                }
            }
        }.collect {
            withContext(Dispatchers.Main) {
                try {
                    img.setImageURI(Uri.parse(it.absolutePath))
                } catch (ex: Exception) {
                    Logger.log("Utils:loadImage", ex.toString())
                }
            }
        }
    }
}

fun Int.decimalValue(currency: Currency): Float {
    val d: Float = when(currency.minorUnit) {
        0 -> 1.0f
        1 -> 10.0f
        2 -> 100.0f
        3 -> 1000.0f
        4 -> 10000.0f
        else -> error("Currency Minor is more than value is supported")
    }
    return this / d
}

fun Int.formattedAmount(currency: Currency): String {
    return this.decimalValue(currency).formattedAmount(currency)
}

fun Float.formattedAmount(currency: Currency?): String {
    val am = String.format("%.2f", this)
    return when(currency) {
        Currency.TJS -> "$am сом."
        Currency.RUB -> "₽$am"
        else -> "${currency?.symbol}$am"
    }
}

fun Double.formattedAmount(currency: Currency?): String {
    return this.toFloat().formattedAmount(currency)
}

fun getProgressBg(bgColor: Int = Color.GRAY, fillColor: Int = Color.GREEN, radius: Float = Theme.dp(12f).toFloat()): Drawable {
    val round = RoundRectShape(floatArrayOf(radius, radius, radius, radius, radius, radius, radius, radius), null, null)
    val bgShape = ShapeDrawable().apply { this.shape = round }
    bgShape.paint.style = Paint.Style.FILL
    bgShape.paint.color = bgColor
    val fillShape = ShapeDrawable().apply { this.shape = round }
    fillShape.paint.style = Paint.Style.FILL
    fillShape.paint.color = fillColor
    val clipDrawable = ClipDrawable(fillShape, Gravity.LEFT, ClipDrawable.HORIZONTAL)
    return LayerDrawable(arrayOf(bgShape, clipDrawable))
}

fun checkEmptyLyt(parent: LinearLayout, tv: View) {
    if (parent.childCount == 0) {
        tv.visibility = View.GONE
        parent.visibility = View.GONE
    } else {
        tv.visibility = View.VISIBLE
        parent.visibility = View.VISIBLE
    }
}

fun isCoreConnected(activity: Activity): Boolean {
    return if (NCPCore.instance.network.isConnected) {
        true
    } else {
        Toast.makeText(activity, nexus.appui.R.string.not_connect_server, Toast.LENGTH_SHORT).show()
        false
    }
}

fun btnSelector(focusedColor: Int = Theme.getColor(Theme.key_primaryColor), defaultColor: Int = Theme.getColor(Theme.key_primaryColor), radius: Int = dp(10f)): StateListDrawable {
    val res = StateListDrawable()
    res.addState(intArrayOf(android.R.attr.state_pressed), ViewUtils.getRect(focusedColor, radius))
    res.addState(intArrayOf(android.R.attr.state_selected), ViewUtils.getRect(focusedColor, radius))
    res.addState(intArrayOf(), ViewUtils.getRect(defaultColor, radius))
    return res
}