package nexus.billing.ui.plan

import android.animation.LayoutTransition
import android.content.res.ColorStateList
import android.graphics.PorterDuff
import android.os.Bundle
import android.text.Editable
import android.text.TextWatcher
import android.util.TypedValue
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.EditText
import android.widget.FrameLayout
import android.widget.LinearLayout
import android.widget.TextView
import androidx.core.view.setPadding
import androidx.core.widget.TextViewCompat
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import androidx.transition.TransitionInflater
import com.google.android.material.appbar.MaterialToolbar
import io.reactivex.rxjava3.android.schedulers.AndroidSchedulers
import io.reactivex.rxjava3.core.Observable
import io.reactivex.rxjava3.schedulers.Schedulers
import nexus.appui.Theme
import nexus.appui.Theme.dp
import nexus.appui.recycler.SpaceItemDecoration
import nexus.appui.ui.BaseFragment
import nexus.appui.utils.ViewUtils
import nexus.billing.R
import nexus.billing.models.PlanInfo
import nexus.billing.ui.rows.RowPlanCountry
import java.util.Locale
import java.util.concurrent.TimeUnit

class SelectCountryFragment: BaseFragment() {

    companion object {
        const val TAG = "SelectCountryFragment"
        fun getInstance(categoryId: String, publicPlans: ArrayList<PlanInfo>): SelectCountryFragment {
            val fragment = SelectCountryFragment()
            fragment.publicPlans = publicPlans
            fragment.baseTag = TAG
            val args = Bundle()
            args.putString("categoryId", categoryId)
            fragment.arguments = args
            return fragment
        }
    }

    private lateinit var rv: RecyclerView
    private lateinit var adapter: CountryAdapter
    private var publicPlans = ArrayList<PlanInfo>()

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        val inflater = TransitionInflater.from(requireContext())
        enterTransition = inflater.inflateTransition(R.transition.slide_right)
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View {
        val countries = this.publicPlans.filter { it.category.id == arguments?.getString("categoryId") }
            .distinctBy { it.provider.country }.map { it.provider.country }.sorted()

        val context = requireContext()
        val rootView = FrameLayout(context)
        rootView.fitsSystemWindows = true
        rootView.isClickable = true
        rootView.setBackgroundColor(theme.getColor(Theme.key_mainBackground))

        val contentVertical = LinearLayout(context)
        contentVertical.orientation = LinearLayout.VERTICAL
        contentVertical.layoutTransition = LayoutTransition()
        rootView.addView(contentVertical, ViewUtils.frameMatchMatch())

        this.toolbar = MaterialToolbar(context)
        contentVertical.addView(this.toolbar, ViewUtils.frameMatchWrap())
        this.toolbar.setBackgroundColor(theme.getColor(Theme.key_actionBar))
        this.toolbar.setTitleTextColor(theme.getColor(Theme.key_actionBarText))
        this.toolbar.setTitleTextAppearance(context, nexus.appui.R.style.BigTitleToolbar)
        this.setTitle(R.string.country)
        this.toolbar.elevation = dp(5f).toFloat()
        this.toolbar.setNavigationIcon(R.drawable.ic_arrow_back)
        this.toolbar.navigationIcon?.setColorFilter(theme.getColor(Theme.key_actionBarText), PorterDuff.Mode.SRC_IN)
        this.toolbar.setNavigationOnClickListener {
            requireActivity().supportFragmentManager.popBackStack()
        }

        val descTv = TextView(context)
        contentVertical.addView(descTv, ViewUtils.linearMatchWrap())
        descTv.text = getString(R.string.select_country_desc)
        descTv.setTextColor(theme.getColor(Theme.key_greyColor))
        descTv.typeface = Theme.font
        descTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.secondaryText)
        descTv.setPadding(dp(30f), dp(10f), dp(30f), dp(10f))

        val edtSearch = EditText(context)
        edtSearch.hint = getString(R.string.search_desc)
        edtSearch.isSingleLine = true
        edtSearch.setCompoundDrawablesWithIntrinsicBounds(R.drawable.ic_search, 0, 0, 0)
        TextViewCompat.setCompoundDrawableTintList(edtSearch, ColorStateList.valueOf(theme.getColor(Theme.key_greyColor)))
        edtSearch.compoundDrawablePadding = dp(10f)
        edtSearch.elevation = dp(5f).toFloat()
        edtSearch.setHintTextColor(theme.getColor(Theme.key_greyColor))
        edtSearch.setTextColor(theme.getColor(Theme.key_rowTextBlack))
        edtSearch.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.normalText)
        edtSearch.setPadding(dp(10f))
        edtSearch.background = ViewUtils.getRect(theme.getColor(Theme.key_blockView), dp(10f))
        val eLp = ViewUtils.linearMatchWrap()
        eLp.setMargins(dp(20f), 0, dp(20f), dp(20f))
        contentVertical.addView(edtSearch, eLp)
        Observable.create { subscriber ->
            edtSearch.addTextChangedListener(object : TextWatcher {
                override fun beforeTextChanged(s: CharSequence?, start: Int, count: Int, after: Int) {}
                override fun onTextChanged(s: CharSequence, start: Int, before: Int, count: Int) {
                    subscriber.onNext(s.toString())
                }
                override fun afterTextChanged(s: Editable?) {}
            })
        }.debounce(250, TimeUnit.MILLISECONDS)
            .distinctUntilChanged()
            .subscribeOn(Schedulers.io())
            .observeOn(AndroidSchedulers.mainThread())
            .subscribe { text ->
                val newCountries = if (text.trim().isEmpty()) {
                    countries
                } else {
                    countries.filter { Locale(it, it).displayCountry.contains(text.trim(), true) }
                }
                this.adapter.countries = newCountries
                this.adapter.notifyDataSetChanged()
            }

        this.rv = RecyclerView(context)
        val rvLp = ViewUtils.linearMatchWrap()
        rvLp.setMargins(dp(20f), 0, dp(20f), 0)
        contentVertical.addView(rv, rvLp)
        this.rv.background = ViewUtils.getRect(theme.getColor(Theme.key_blockView), dp(10f))
        this.rv.elevation = dp(5f).toFloat()
        this.rv.layoutManager = LinearLayoutManager(context)
        this.rv.addItemDecoration(SpaceItemDecoration(0))

        this.adapter = CountryAdapter(countries)
        this.rv.adapter = adapter
        return rootView
    }

    private inner class CountryAdapter(var countries: List<String>) : RecyclerView.Adapter<CountryAdapter.CountryHolder>() {
        override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): CountryHolder {
            return CountryHolder(RowPlanCountry(parent.context))
        }

        override fun onBindViewHolder(holder: CountryHolder, position: Int) {
            val country = countries[position]
            val item = holder.itemView as RowPlanCountry
            item.setCountry(country)
            holder.itemView.setOnClickListener {
                val providers = this@SelectCountryFragment.publicPlans.filter { it.provider.country == country }.distinctBy { it.provider.id }
                if (providers.size == 1) {
                    val plans = this@SelectCountryFragment.publicPlans.filter { it.provider.id == providers.firstOrNull()?.provider?.id }
                    pushFragment(RegisterPlansFragment.getInstance(ArrayList(plans)))
                } else {
                    pushFragment(SelectProviderFragment.getInstance(country, publicPlans))
                }
            }
        }

        override fun getItemCount(): Int {
            return countries.size
        }

        inner class CountryHolder(itemView: View) : RecyclerView.ViewHolder(itemView)
    }
}