package nexus.billing.ui.plan

import android.animation.LayoutTransition
import android.content.res.ColorStateList
import android.graphics.PorterDuff
import android.os.Bundle
import android.text.Editable
import android.text.TextWatcher
import android.transition.TransitionInflater
import android.util.TypedValue
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.EditText
import android.widget.FrameLayout
import android.widget.LinearLayout
import android.widget.TextView
import androidx.core.view.setPadding
import androidx.core.widget.TextViewCompat
import androidx.fragment.app.viewModels
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.google.android.material.appbar.MaterialToolbar
import io.reactivex.rxjava3.android.schedulers.AndroidSchedulers
import io.reactivex.rxjava3.core.Observable
import io.reactivex.rxjava3.disposables.CompositeDisposable
import io.reactivex.rxjava3.schedulers.Schedulers
import nexus.appui.Theme
import nexus.appui.Theme.dp
import nexus.appui.recycler.SpaceItemDecoration
import nexus.appui.ui.BaseFragment
import nexus.appui.utils.ViewUtils
import nexus.billing.R
import nexus.billing.methods.common.Provider
import nexus.billing.models.DescriptionObject
import nexus.billing.models.PlanInfo
import nexus.billing.ui.rows.RowProvider
import java.util.concurrent.TimeUnit

class SelectProviderFragment: BaseFragment() {

    companion object {
        const val TAG = "SelectProviderFragment"
        fun getInstance(country: String, publicPlans: ArrayList<PlanInfo>): SelectProviderFragment {
            val fragment = SelectProviderFragment()
            fragment.publicPlans = publicPlans
            fragment.baseTag = TAG
            val args = Bundle()
            args.putString("country", country)
            fragment.arguments = args
            return fragment
        }
    }

    private lateinit var rv: RecyclerView
    private lateinit var adapter: OperatorAdapter
    private var publicPlans = ArrayList<PlanInfo>()
    internal val viewModel: SelectProviderViewModel by viewModels()
    private val bag = CompositeDisposable()

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        val inflater = TransitionInflater.from(requireContext())
        enterTransition = inflater.inflateTransition(R.transition.slide_right)
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View {
        val providers = publicPlans.filter { it.provider.country == arguments?.getString("country") }.distinctBy { it.provider.id }.map { planInfo ->
            Pair(planInfo.provider, planInfo.providerDesc)
        }.sortedBy { it.first.country }

        val context = requireContext()
        val rootView = FrameLayout(context)
        rootView.fitsSystemWindows = true
        rootView.isClickable = true
        rootView.setBackgroundColor(theme.getColor(Theme.key_mainBackground))

        val contentVertical = LinearLayout(context)
        contentVertical.orientation = LinearLayout.VERTICAL
        contentVertical.layoutTransition = LayoutTransition()
        rootView.addView(contentVertical, ViewUtils.frameMatchMatch())

        this.toolbar = MaterialToolbar(context)
        contentVertical.addView(this.toolbar, ViewUtils.frameMatchWrap())
        this.toolbar.setBackgroundColor(theme.getColor(Theme.key_actionBar))
        this.toolbar.setTitleTextColor(theme.getColor(Theme.key_actionBarText))
        this.toolbar.setTitleTextAppearance(context, nexus.appui.R.style.BigTitleToolbar)
        this.setTitle(R.string.select_operator)
        this.toolbar.elevation = dp(5f).toFloat()
        this.toolbar.setNavigationIcon(R.drawable.ic_arrow_back)
        this.toolbar.navigationIcon?.setColorFilter(theme.getColor(Theme.key_actionBarText), PorterDuff.Mode.SRC_IN)
        this.toolbar.setNavigationOnClickListener {
            requireActivity().supportFragmentManager.popBackStack()
        }

        val descTv = TextView(context)
        contentVertical.addView(descTv, ViewUtils.linearMatchWrap())
        descTv.text = getString(R.string.select_operator_desc)
        descTv.setTextColor(theme.getColor(Theme.key_greyColor))
        descTv.typeface = Theme.font
        descTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.secondaryText)
        descTv.setPadding(dp(20f), dp(10f), dp(20f), dp(10f))

        val edtSearch = EditText(context)
        edtSearch.hint = getString(R.string.search_operator)
        edtSearch.setHintTextColor(theme.getColor(Theme.key_greyColor))
        edtSearch.setTextColor(theme.getColor(Theme.key_rowTextBlack))
        edtSearch.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.normalText)
        edtSearch.isSingleLine = true
        edtSearch.setPadding(dp(10f))
        edtSearch.setCompoundDrawablesWithIntrinsicBounds(R.drawable.ic_search, 0, 0, 0)
        TextViewCompat.setCompoundDrawableTintList(edtSearch, ColorStateList.valueOf(theme.getColor(Theme.key_greyColor)))
        edtSearch.compoundDrawablePadding = dp(10f)
        edtSearch.elevation = dp(5f).toFloat()
        edtSearch.background = ViewUtils.getRect(theme.getColor(Theme.key_blockView), dp(10f))
        this.bag.add(Observable.create { subscriber ->
            edtSearch.addTextChangedListener(object : TextWatcher {
                override fun beforeTextChanged(s: CharSequence?, start: Int, count: Int, after: Int) {}
                override fun onTextChanged(s: CharSequence, start: Int, before: Int, count: Int) {
                    subscriber.onNext(s.toString())
                }
                override fun afterTextChanged(s: Editable?) {}
            })
        }.debounce(250, TimeUnit.MILLISECONDS)
            .distinctUntilChanged()
            .subscribeOn(Schedulers.io())
            .observeOn(AndroidSchedulers.mainThread())
            .subscribe { text ->
                val newProviders = if (text.trim().isEmpty()) {
                    providers
                } else {
                    providers.filter { (it.second?.title?.contains(text.trim(), true) ?: false) || (it.second?.shortText?.contains(text.trim(), true) ?: false) }
                }
                this.adapter.providers = newProviders
                this.adapter.notifyDataSetChanged()
            })

        val eLp = ViewUtils.linearMatchWrap()
        eLp.setMargins(dp(20f), 0, dp(20f), dp(20f))
        contentVertical.addView(edtSearch, eLp)
        this.rv = RecyclerView(context)
        val rvLp = ViewUtils.linearMatchWrap()
        rvLp.setMargins(dp(20f), 0, dp(20f), 0)
        contentVertical.addView(this.rv, rvLp)
        this.rv.background = ViewUtils.getRect(theme.getColor(Theme.key_blockView), dp(10f))
        this.rv.elevation = dp(5f).toFloat()
        this.rv.layoutManager = LinearLayoutManager(context)
        this.rv.addItemDecoration(SpaceItemDecoration(0))

        this.adapter = OperatorAdapter(providers)
        this.rv.adapter = this.adapter

        return rootView
    }

    override fun onDestroy() {
        this.bag.clear()
        super.onDestroy()
    }

    inner class OperatorAdapter(var providers: List<Pair<Provider, DescriptionObject?>>) : RecyclerView.Adapter<OperatorAdapter.ViewHolder>() {
        override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): ViewHolder {
            return ViewHolder(RowProvider(parent.context))
        }

        override fun onBindViewHolder(holder: ViewHolder, position: Int) {
            val provider = this.providers[position]
            val item = holder.itemView as RowProvider
            item.showProvider(provider)
            holder.itemView.setOnClickListener {
                val plans = publicPlans.filter { it.provider.id == provider.first.id }
                pushFragment(RegisterPlansFragment.getInstance(ArrayList(plans)))
            }
        }

        override fun getItemCount(): Int {
            return this.providers.size
        }

        inner class ViewHolder(itemView: View) : RecyclerView.ViewHolder(itemView)
    }
}