package nexus.billing.ui.profile

import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import androidx.lifecycle.liveData
import io.reactivex.rxjava3.disposables.CompositeDisposable
import io.reactivex.rxjava3.kotlin.addTo
import nexus.appui.utils.Resource
import nexus.billing.billingKit
import nexus.billing.methods.common.Route
import nexus.billing.methods.common.Verification
import nexus.billing.models.PlanObject
import nexus.core.NCPCore
import nexus.core.util.NotifyType

class ProfilePlanViewViewModel: ViewModel() {

    private var routes = HashMap<String, Array<Route>>()
    var profilePlan = MutableLiveData<PlanObject>()
    var actionSelectAssets: Boolean? = null
    var actionVerify: Verification? = null
    val bag = CompositeDisposable()

    private var profilePlanId: String? = null
        set(value) {
            field = value
            if (value != null)  {
                NCPCore.instance.billingKit.planChangeEvent.subscribe { notify ->
                    if (notify.value.id == this.profilePlanId) {
                        when(notify.type) {
                            NotifyType.Create, NotifyType.Update -> {
                                this.profilePlan.postValue(notify.value)
                            }
                            NotifyType.Delete -> {
                                this.profilePlan.postValue(null)
                            }
                        }
                    }
                }.addTo(this.bag)
            }
        }

    fun loadPlan(profilePlanId: String): LiveData<Resource<Boolean>>  {
        this.profilePlanId = profilePlanId
        return liveData {
            emit(Resource.loading(null))
            try {
                val result = NCPCore.instance.billingKit.controller.getProfilePlan(profilePlanId)
                if (result != null) {
                    this@ProfilePlanViewViewModel.profilePlan.postValue(result)
                    emit(Resource.success(true))
                } else {
                    emit(Resource.error("not found", null))
                }
            } catch (ex: Exception) {
                emit(Resource.error(ex.message ?: ex.toString(), null))
            }
        }
    }

    fun deletePlan(profilePlanId: String): LiveData<Resource<Boolean>> {
        return liveData {
            emit(Resource.loading(null))
            try {
                val result = NCPCore.instance.billingKit.controller.requestPlanRemove(profilePlanId)
                if (result) {
                    emit(Resource.success(true))
                } else {
                    emit(Resource.error("not deleted", null))
                }
            } catch (ex: Exception) {
                emit(Resource.error(ex.message ?: ex.toString(), null))
            }
        }
    }

    fun activatePlan(profilePlanId: String): LiveData<Resource<PlanObject>> {
        return liveData {
            emit(Resource.loading(null))
            try {
                val result = NCPCore.instance.billingKit.controller.requestPlanActivate(profilePlanId)
                if (result != null) {
                    this@ProfilePlanViewViewModel.profilePlan.postValue(result)
                    emit(Resource.success(result))
                } else {
                    emit(Resource.error("not activated", null))
                }
            } catch (ex: Exception) {
                emit(Resource.error(ex.message ?: ex.toString(), null))
            }
        }
    }

    fun updateProfilePlan(): LiveData<Resource<Boolean>> {
        return liveData {
            try {
                emit(Resource.loading(false))
                this@ProfilePlanViewViewModel.profilePlanId?.let {
                    NCPCore.instance.billingKit.controller.requestGetProfilePlan(it)?.let { plan ->
                        this@ProfilePlanViewViewModel.profilePlan.postValue(plan)
                        emit(Resource.success(true))
                    } ?: run {
                        emit(Resource.error("Plan not found", false))
                    }
                }
            } catch (ex: Exception) {
                emit(Resource.error(ex.toString(), false))
            }
        }
    }

    fun setForwarding(forwarding: Boolean, serviceId: String): LiveData<Resource<Boolean>> {
        return liveData {
            try {
                emit(Resource.loading(false))
                this@ProfilePlanViewViewModel.profilePlanId?.let {
                    NCPCore.instance.billingKit.controller.requestServiceForwarding(it, serviceId, forwarding)?.let { service ->
                        emit(Resource.success(true))
                    } ?: run {
                        emit(Resource.error("Error", false))
                    }
                }
            } catch (ex: Exception) {
                emit(Resource.error(ex.toString(), false))
            }
        }
    }

    suspend fun getPlanRoutes(planId: String, serviceId: String): Array<Route> {
        return this.routes[serviceId]?.let {
            it
        } ?: run {
            val newRoutes = NCPCore.instance.billingKit.controller.requestPlanServiceRoutes(planId, serviceId)
            this.routes[serviceId] = newRoutes
            newRoutes
        }
    }

    fun activateService(profilePlanId: String, serviceId: String): LiveData<Resource<PlanObject>> {
        return liveData {
            emit(Resource.loading(null))
            try {
                val result = NCPCore.instance.billingKit.controller.requestPlanServiceActivate(profilePlanId, serviceId)
                if (result != null) {
                    emit(Resource.success(result))
                } else {
                    emit(Resource.error("error activate service", null))
                }
            } catch (ex: Exception) {
                emit(Resource.error(ex.message ?: ex.toString(), null))
            }
        }
    }

    fun removeService(profilePlanId: String, serviceId: String): LiveData<Resource<PlanObject>> {
        return liveData {
            emit(Resource.loading(null))
            try {
                val result = NCPCore.instance.billingKit.controller.requestPlanServiceRemove(profilePlanId, serviceId)
                if (result != null) {
                    emit(Resource.success(result))
                } else {
                    emit(Resource.error("error remove service", null))
                }
            } catch (ex: Exception) {
                emit(Resource.error(ex.message ?: ex.toString(), null))
            }
        }
    }
}