package nexus.billing.ui.profile

import android.animation.LayoutTransition
import android.graphics.PorterDuff
import android.os.Bundle
import android.os.Handler
import android.transition.TransitionInflater
import android.util.TypedValue
import android.view.Gravity
import android.view.LayoutInflater
import android.view.MenuItem
import android.view.View
import android.view.ViewGroup
import android.widget.FrameLayout
import android.widget.LinearLayout
import android.widget.ProgressBar
import android.widget.TextView
import androidx.fragment.app.viewModels
import androidx.recyclerview.widget.DiffUtil
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.ListAdapter
import androidx.recyclerview.widget.RecyclerView
import androidx.swiperefreshlayout.widget.SwipeRefreshLayout
import com.google.android.material.appbar.MaterialToolbar
import nexus.appui.Theme
import nexus.appui.Theme.dp
import nexus.appui.ui.BaseFragment
import nexus.appui.utils.Resource
import nexus.appui.utils.ViewUtils
import nexus.appui.utils.setDebounceClickListener
import nexus.billing.R
import nexus.billing.billingKit
import nexus.billing.methods.profile.ProfilePlan
import nexus.billing.models.PlanObject
import nexus.billing.ui.ProfilePlansActivity
import nexus.billing.ui.plan.AddPlanFragment
import nexus.billing.ui.rows.RowProfilePlan
import nexus.core.NCPCore
import nexus.core.methods.access.EndpointViewPermission.ViewName.BILLING_EXTERNAL_REGISTER_VIEW

class ProfilePlansFragment(private val showBackButton: Boolean = true, private val animate: Boolean = true): BaseFragment() {

    companion object {
        const val TAG = "ProfilePlansFragment"
        fun getInstance(showBackButton: Boolean = true, animate: Boolean = true): ProfilePlansFragment {
            val fragment = ProfilePlansFragment(showBackButton, animate)
            fragment.baseTag = TAG
            val args = Bundle()
            fragment.arguments = args
            return fragment
        }
    }

    private val viewModel: ProfilePlansViewModel by viewModels()
    private var adapter: ProfilePlanAdapter? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        if (this.animate) {
            val inflater = TransitionInflater.from(requireContext())
            enterTransition = inflater.inflateTransition(R.transition.slide_right)
        }
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View {
        val context = requireContext()

        val rootView = FrameLayout(context)
        rootView.fitsSystemWindows = true
        rootView.isClickable = true
        rootView.setBackgroundColor(theme.getColor(Theme.key_mainBackground))

        val contentVertical = LinearLayout(context)
        contentVertical.orientation = LinearLayout.VERTICAL
        contentVertical.layoutTransition = LayoutTransition()
        rootView.addView(contentVertical, ViewUtils.frameMatchMatch())

        this.toolbar = MaterialToolbar(context)
        contentVertical.addView(this.toolbar, ViewUtils.frameMatchWrap())
        this.toolbar.setBackgroundColor(theme.getColor(Theme.key_actionBar))
        this.toolbar.setTitleTextColor(theme.getColor(Theme.key_actionBarText))
        this.toolbar.setSubtitleTextColor(theme.getColor(Theme.key_actionBarText))
        this.toolbar.setTitleTextAppearance(context, nexus.appui.R.style.BigTitleToolbar)
        this.toolbar.elevation = dp(5f).toFloat()
        this.setTitle(R.string.plans)
        this.toolbar.menu?.add(0, 55, 0, R.string.add)?.setIcon(ViewUtils.getMenuIcon(R.drawable.ic_add, requireContext()))?.setShowAsAction(MenuItem.SHOW_AS_ACTION_ALWAYS)
        if (this.showBackButton) {
            this.toolbar.setNavigationIcon(R.drawable.ic_arrow_back)
            this.toolbar.navigationIcon?.setColorFilter(theme.getColor(Theme.key_actionBarText), PorterDuff.Mode.SRC_IN)
            this.toolbar.setNavigationOnClickListener {
                if (activity is ProfilePlansActivity) {
                    activity?.finish()
                } else {
                    requireActivity().supportFragmentManager.popBackStack()
                }
            }
        }
        this.toolbar.setOnMenuItemClickListener {
            if (NCPCore.instance.currentAccount?.hasPermission(BILLING_EXTERNAL_REGISTER_VIEW) == true) {
                RegisterProfilePlanDialog.getInstance(requireActivity()).show(null)
            } else {
                pushFragment(AddPlanFragment.getInstance())
            }
            return@setOnMenuItemClickListener true
        }

        val swipe = SwipeRefreshLayout(requireContext())
        contentVertical.addView(swipe, ViewUtils.linearMatchMatch())

        val rv = RecyclerView(requireContext())
        swipe.addView(rv, ViewUtils.frameMatchMatch())
        rv.layoutManager = LinearLayoutManager(requireContext())
        this.adapter = ProfilePlanAdapter(this)
        rv.adapter = this.adapter

        val progress = ProgressBar(requireContext())
        rootView.addView(progress, ViewUtils.frameWrapWrap(Gravity.CENTER))

        val emptyTv = TextView(this.context)
        emptyTv.text = getString(R.string.hannt_plan)
        emptyTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.normalText)
        emptyTv.gravity = Gravity.CENTER
        emptyTv.setTextColor(Theme.getColor(Theme.key_greyColor))
        emptyTv.visibility = View.GONE
        rootView.addView(emptyTv, ViewUtils.frameMatchWrap(Gravity.CENTER))

        this.viewModel.profilePlans.observe(viewLifecycleOwner) { result ->
            when(result.status) {
                Resource.Status.LOADING -> {
                    progress.visibility = View.VISIBLE
                }
                Resource.Status.ERROR -> {
                    progress.visibility = View.GONE
                    ViewUtils.showErrorDialog(requireActivity(), null, getString(R.string.request_error_repeat))
                }
                Resource.Status.SUCCESS -> {
                    val list = ArrayList<PlanObject>()
                    result.data?.let {
                        list.addAll(it.filter { it.plan.state == ProfilePlan.State.active })
                        list.addAll(it.filter { it.plan.state == ProfilePlan.State.pending })
                        list.addAll(it.filter { it.plan.state != ProfilePlan.State.active && it.plan.state != ProfilePlan.State.pending })
                    }
                    this.adapter?.submitList(list)
                    progress.visibility = View.GONE
                    emptyTv.visibility = if (list.isEmpty()) View.VISIBLE else View.GONE
                }
            }
        }
        if ((this.viewModel.profilePlans.value?.data?.size ?: 0) == 0) {
            this.viewModel.loadProfilePlans()
        }
        swipe.setOnRefreshListener {
            if (this.viewModel.profilePlans.value?.status != Resource.Status.LOADING) {
                NCPCore.instance.billingKit.controller.updateProfilePlans(NCPCore.currentLanguage, true)
            }
            Handler(requireContext().mainLooper).postDelayed({
                swipe.isRefreshing = false
            }, 2500)
        }

        return rootView
    }

    class ProfilePlanAdapter(var fragment: BaseFragment) : ListAdapter<PlanObject, ProfilePlanAdapter.ViewHolder>(DiffCallback) {

        companion object DiffCallback : DiffUtil.ItemCallback<PlanObject>() {
            override fun areItemsTheSame(oldItem: PlanObject, newItem: PlanObject): Boolean {
                return oldItem.id == newItem.id
            }
            override fun areContentsTheSame(oldItem: PlanObject, newItem: PlanObject): Boolean {
                return oldItem.id == newItem.id && oldItem.plan.state == newItem.plan.state && oldItem.plan.actions.size == newItem.plan.actions.size
                        && oldItem.plan.activatedAt == newItem.plan.activatedAt && oldItem.plan.suspendedAt == newItem.plan.suspendedAt
                        && oldItem.plan.services.size == newItem.plan.services.size && oldItem.price == newItem.price
            }
        }

        override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): ViewHolder {
            return ViewHolder(RowProfilePlan(parent.context))
        }

        override fun onBindViewHolder(holder: ViewHolder, position: Int) {
            val plan = getItem(position)
            holder.showData(plan)
            holder.row.setDebounceClickListener {
                fragment.pushFragment(ProfilePlanViewFragment.getInstance(plan.id))
            }
        }

        inner class ViewHolder(var row: RowProfilePlan) : RecyclerView.ViewHolder(row) {
            fun showData(plan: PlanObject) {
                this.row.showData(plan)
                if (adapterPosition == 0 || getItem(adapterPosition - 1).plan.state != plan.plan.state) {
                    row.stateTv.text = getStateName(plan.plan.state)
                    row.stateTv.visibility = View.VISIBLE
                } else {
                    row.stateTv.visibility = View.GONE
                }
            }
        }

        private fun getStateName(state: ProfilePlan.State): String {
            val context = fragment.requireContext()
            return when(state) {
                ProfilePlan.State.active -> context.getString(R.string.active)
                ProfilePlan.State.pending -> context.getString(R.string.plan_pending)
                ProfilePlan.State.suspended -> context.getString(R.string.suspended)
                ProfilePlan.State.removed -> context.getString(R.string.removed)
                ProfilePlan.State.inactive -> context.getString(R.string.inactive)
            }
        }
    }
}