package nexus.billing.ui.profile

import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import androidx.lifecycle.liveData
import androidx.lifecycle.switchMap
import androidx.lifecycle.viewModelScope
import io.reactivex.rxjava3.disposables.CompositeDisposable
import io.reactivex.rxjava3.kotlin.addTo
import kotlinx.coroutines.launch
import nexus.appui.utils.Resource
import nexus.billing.billingKit
import nexus.billing.methods.profile.ProfilePlan
import nexus.billing.models.PlanInfo
import nexus.billing.models.PlanObject
import nexus.core.NCPCore
import nexus.core.util.NotifyType

class ProfilePlansViewModel: ViewModel() {

    private val plansTrigger = MutableLiveData<String>()
    val profilePlans = MutableLiveData<Resource<ArrayList<PlanObject>>>()
    val bag = CompositeDisposable()

    init {
        NCPCore.instance.billingKit.planChangeEvent.subscribe { notify ->
            if (notify.type == NotifyType.Delete || notify.value.plan.state == ProfilePlan.State.removed) {
                val list = this@ProfilePlansViewModel.profilePlans.value?.data ?: ArrayList()
                val index = list.indexOfFirst { it.id == notify.value.id }
                if (index != -1) {
                    list.removeAt(index)
                }
                this@ProfilePlansViewModel.profilePlans.postValue(Resource.success(list))
            } else {
                profilePlanUpdated(notify.value)
            }
        }.addTo(this.bag)
    }

    val plans: LiveData<Resource<ArrayList<PlanInfo>>> = plansTrigger.switchMap {
        liveData {
            emit(Resource.loading(null))
            try {
                val plans = NCPCore.instance.billingKit.controller.requestAllPlans(NCPCore.currentLanguage)
                emit(Resource.success(plans))
            } catch (ex: Exception) {
                emit(Resource.error(ex.message ?: ex.toString(), null))
            }
        }
    }

    private fun profilePlanUpdated(profilePlan: PlanObject) {
        val list = this.profilePlans.value?.data ?: ArrayList()
        val index = list.indexOfFirst { it.id == profilePlan.id }
        if (index == -1) {
            list.add(profilePlan)
        } else {
            list[index] = profilePlan
        }
        this.profilePlans.postValue(Resource.success(list))
    }

    fun loadProfilePlans() {
        this.viewModelScope.launch {
            this@ProfilePlansViewModel.profilePlans.postValue(Resource.loading(null))
            try {
                val plans = NCPCore.instance.billingKit.controller.getProfilePlans(NCPCore.currentLanguage)
                    .filter { it.plan.state != ProfilePlan.State.removed }
                this@ProfilePlansViewModel.profilePlans.postValue(Resource.success(ArrayList(plans)))
                if (plans.isNotEmpty()) {
                    NCPCore.instance.billingKit.controller.updateProfilePlans(NCPCore.currentLanguage, false)
                }
            } catch (ex: Exception) {
                this@ProfilePlansViewModel.profilePlans.postValue(Resource.error(ex.message ?: ex.toString(), null))
            }
        }
    }

    fun loadPlans() {
        this.plansTrigger.value = ""
    }

    override fun onCleared() {
        super.onCleared()
        this.bag.clear()
    }
}