package nexus.billing.ui.profile

import android.graphics.Typeface
import android.util.TypedValue
import android.view.Gravity
import android.view.View
import android.view.ViewTreeObserver.OnScrollChangedListener
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.ProgressBar
import android.widget.ScrollView
import android.widget.TextView
import androidx.core.view.setMargins
import com.google.android.material.bottomsheet.BottomSheetBehavior
import com.google.android.material.bottomsheet.BottomSheetDialog
import io.reactivex.rxjava3.disposables.CompositeDisposable
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext
import nexus.appui.Theme
import nexus.appui.ui.BaseDialog
import nexus.appui.ui.BaseFragment
import nexus.appui.utils.ViewUtils
import nexus.appui.utils.setDebounceClickListener
import nexus.billing.R
import nexus.billing.billingKit
import nexus.billing.methods.requests.profileplan.ProfilePlanRoute
import nexus.billing.models.PlanInfo
import nexus.billing.models.PlanObject
import nexus.billing.ui.plan.PublicPlanViewFragment
import nexus.billing.ui.rows.RowSelectPlanView
import nexus.core.NCPCore
import nexus.core.util.Logger
import kotlin.math.max
import kotlin.math.min


class SelectPlanDialog(val fragment: BaseFragment, var type: ProfilePlanRoute.RouteType, var phoneNumber: String): BaseDialog() {

    companion object {
        @JvmStatic
        fun getInstance(fragment: BaseFragment, type: ProfilePlanRoute.RouteType, phoneNumber: String): SelectPlanDialog {
            return SelectPlanDialog(fragment, type, phoneNumber)
        }
    }

    interface SelectPlanListener {
        fun planSelected(planObject: PlanObject)
    }

    private var bottomDialog: BottomSheetDialog? = null
    private var listener: SelectPlanListener? = null
    private var progressBar: ProgressBar

    private val myTariffTv: TextView
    private val myTariffLyt: LinearLayout

    private val actionPlanTv: TextView
    private val actionPlansLyt: LinearLayout

    private val recommendTv: TextView
    private val recommendInfoLyt: LinearLayout
    private val recommendLyt: LinearLayout

    private val scrollView: ScrollView
    private val infoImg: ImageView

    private val bag = CompositeDisposable()

    init {
        val activity = this.fragment.requireActivity()
        val font = Theme.font
        this.bottomDialog = BottomSheetDialog(activity, nexus.appui.R.style.SheetDialog)

        this.scrollView = ScrollView(activity)
        val contentLyt = LinearLayout(activity)
        this.scrollView.addView(contentLyt, ViewUtils.scrollMatchWrap())
        contentLyt.background = ViewUtils.getRect(Theme.getColor(Theme.key_mainBackground), Theme.dp(15f), Theme.dp(15f), 0, 0)
        contentLyt.orientation = LinearLayout.VERTICAL
//        contentLyt.layoutTransition = LayoutTransition().apply { setAnimateParentHierarchy(false) }
        contentLyt.setPadding(0, 0, 0, Theme.dp(50f))

        val titleTv = TextView(activity)
        contentLyt.addView(titleTv, ViewUtils.linearMatchWrap())
        titleTv.setTypeface(font, Typeface.BOLD)
        titleTv.gravity = Gravity.CENTER
        titleTv.setText(if (this.type == ProfilePlanRoute.RouteType.voice) R.string.calls else R.string.sms)
        titleTv.setPadding(0, Theme.dp(10f), 0, Theme.dp(10f))
        titleTv.setTextColor(Theme.getColor(Theme.key_primaryColor))
        titleTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.titleText)

        this.recommendInfoLyt = LinearLayout(activity)
        this.recommendInfoLyt.gravity = Gravity.CENTER_HORIZONTAL
        this.recommendInfoLyt.setPadding(Theme.dp(20f), Theme.dp(10f), Theme.dp(20f), Theme.dp(10f))
        this.recommendInfoLyt.orientation = LinearLayout.VERTICAL
        contentLyt.addView(this.recommendInfoLyt, ViewUtils.linearMatchWrap())

        this.infoImg = ImageView(activity)
        this.recommendInfoLyt.addView(this.infoImg, ViewUtils.linearParams(120, 120))
        this.infoImg.setImageResource(R.drawable.select_plan_image)
//        this.scrollView.viewTreeObserver.addOnScrollChangedListener(ScrollPositionObserver())

        val descTv = TextView(activity)
        this.recommendInfoLyt.addView(descTv, ViewUtils.linearMatchWrap())
        descTv.setText(if (this.type == ProfilePlanRoute.RouteType.voice) R.string.select_call_plan_info else R.string.select_sms_plan_info)
        descTv.setPadding(0, Theme.dp(10f), 0, 0)
        descTv.setTextColor(Theme.getColor(Theme.key_rowTextBlack))
        descTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.normalText)

        this.myTariffTv = TextView(activity)
        contentLyt.addView(this.myTariffTv, ViewUtils.linearMatchWrap())
        this.myTariffTv.setTypeface(font, Typeface.BOLD)
        this.myTariffTv.visibility = View.GONE
        this.myTariffTv.setText(R.string.your_tariff_plans)
        this.myTariffTv.setPadding(Theme.dp(20f), Theme.dp(10f), Theme.dp(20f), Theme.dp(10f))
        this.myTariffTv.setTextColor(Theme.getColor(Theme.key_primaryColor))
        this.myTariffTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.titleText)

        this.myTariffLyt = LinearLayout(activity)
        this.myTariffLyt.orientation = LinearLayout.VERTICAL
        contentLyt.addView(this.myTariffLyt)

        this.actionPlanTv = TextView(activity)
        contentLyt.addView(this.actionPlanTv, ViewUtils.linearMatchWrap())
        this.actionPlanTv.setTypeface(font, Typeface.BOLD)
        this.actionPlanTv.visibility = View.GONE
        this.actionPlanTv.setText(R.string.action_required)
        this.actionPlanTv.setPadding(Theme.dp(20f), Theme.dp(10f), Theme.dp(20f), Theme.dp(10f))
        this.actionPlanTv.setTextColor(Theme.getColor(Theme.key_primaryColor))
        this.actionPlanTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.titleText)

        this.actionPlansLyt = LinearLayout(activity)
        this.actionPlansLyt.orientation = LinearLayout.VERTICAL
        contentLyt.addView(this.actionPlansLyt)

        this.recommendTv = TextView(activity)
        contentLyt.addView(this.recommendTv, ViewUtils.linearMatchWrap())
        this.recommendTv.setTypeface(font, Typeface.BOLD)
        this.recommendTv.text = activity.getString(R.string.recommended)
        this.recommendTv.visibility = View.GONE
        this.recommendTv.setPadding(Theme.dp(20f), Theme.dp(10f), Theme.dp(20f), Theme.dp(10f))
        this.recommendTv.setTextColor(Theme.getColor(Theme.key_primaryColor))
        this.recommendTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.titleText)

        this.recommendLyt = LinearLayout(activity)
        this.recommendLyt.visibility = View.GONE
        this.recommendLyt.orientation = LinearLayout.VERTICAL
        contentLyt.addView(this.recommendLyt)

        this.progressBar = ProgressBar(activity)
        contentLyt.addView(this.progressBar, ViewUtils.linearWrapWrap().apply {
            setMargins(Theme.dp(20f))
            gravity = Gravity.CENTER
        })
        this.loadPlans()

        this.bottomDialog?.setContentView(this.scrollView, ViewUtils.frameMatchWrap())
    }

    private inner class ScrollPositionObserver : OnScrollChangedListener {
        private val startHeight = Theme.dp(150f)

        override fun onScrollChanged() {
            val scrollY = min(max(this@SelectPlanDialog.scrollView.scrollY, 0), startHeight)
            this@SelectPlanDialog.infoImg.translationY = (scrollY / 2).toFloat()

            // alpha you could set to ActionBar background
            val alpha = scrollY / startHeight.toFloat()
        }
    }

    private fun loadPlans() {
        CoroutineScope(Dispatchers.IO).launch {
            try {
                val getRoute = if (this@SelectPlanDialog.phoneNumber.isNotEmpty()) {
                    NCPCore.instance.billingKit.controller.requestProfilePlanRoute(this@SelectPlanDialog.type, this@SelectPlanDialog.phoneNumber, null)
                } else { null }
                val myPlans = NCPCore.instance.billingKit.controller.getProfilePlans(NCPCore.currentLanguage).sortedByDescending { it.plan.state }
                withContext(Dispatchers.Main) {
                    if (this@SelectPlanDialog.bottomDialog?.isShowing == true) {
                        for (myPlan in myPlans) {
                            getRoute?.let { getRoute ->
                                getRoute.routes.find { myPlan.id == it.profilePlanId() }?.let { route ->
                                    this@SelectPlanDialog.showPlan(myPlan, this@SelectPlanDialog.myTariffLyt, route)
                                } ?: run {
                                    if (getRoute.profilePlans?.find { it.id == myPlan.id } != null) {
                                        this@SelectPlanDialog.showPlan(myPlan, this@SelectPlanDialog.actionPlansLyt, action = true)
                                    }
                                }
                            } ?: run {
                                this@SelectPlanDialog.showPlan(myPlan, this@SelectPlanDialog.myTariffLyt, null)
                            }
                        }

                        getRoute?.let {
                            val recommendPlans = ArrayList<PlanInfo>()
                            if (getRoute.plans.isNotEmpty()) {
                                getRoute.plans.forEach {
                                    NCPCore.instance.billingKit.controller.requestGetPlanWithInfo(it.id)?.let { plan -> recommendPlans.add(plan) }
                                }
                                if (recommendPlans.isNotEmpty()) {
                                    withContext(Dispatchers.Main) {
                                        if (this@SelectPlanDialog.bottomDialog?.isShowing == true) {
                                            for (plan in recommendPlans) {
                                                this@SelectPlanDialog.showPlan(plan, this@SelectPlanDialog.recommendLyt)
                                            }
                                        }
                                        this@SelectPlanDialog.recommendLyt.visibility = View.VISIBLE
                                        this@SelectPlanDialog.recommendTv.visibility = View.VISIBLE
                                    }
                                }
                            }
                        }
                    }

                    this@SelectPlanDialog.progressBar.visibility = View.GONE
                    if (this@SelectPlanDialog.myTariffLyt.childCount > 0) {
                        this@SelectPlanDialog.myTariffTv.visibility = View.VISIBLE
                        this@SelectPlanDialog.myTariffLyt.visibility = View.VISIBLE
                    }
                    if (this@SelectPlanDialog.actionPlansLyt.childCount > 0) {
                        this@SelectPlanDialog.actionPlanTv.visibility = View.VISIBLE
                        this@SelectPlanDialog.actionPlansLyt.visibility = View.VISIBLE
                    }
                }
            } catch (e: Exception) {
                Logger.log("SelectPlanDialog:loadPlans", e.toString(), true)
            }
        }
    }

    private fun showPlan(plan: PlanObject, layout: LinearLayout, route: ProfilePlanRoute.Response.Routes? = null, enable: Boolean = true, action: Boolean = false) {
        this.fragment.context?.let {
            val row = RowSelectPlanView(it)
            row.setActive(enable)
            row.showData(plan, this.type, route, action)
            layout.addView(row, ViewUtils.linearMatchWrap().apply { setMargins(Theme.dp(20f), Theme.dp(3f), Theme.dp(20f), Theme.dp(7f)) })
            if (enable) {
                row.setDebounceClickListener {
                    if (action) {
                        this@SelectPlanDialog.fragment.pushFragment(ProfilePlanViewFragment.getInstance(plan.id))
                    } else {
                        this@SelectPlanDialog.listener?.planSelected(plan)
                    }
                    this@SelectPlanDialog.close()
                }
            }
        }
    }

    private fun showPlan(plan: PlanInfo, layout: LinearLayout) {
        this.fragment.context?.let {
            val row = RowSelectPlanView(it)
            row.showData(plan)
            layout.addView(row, ViewUtils.linearMatchWrap().apply { setMargins(Theme.dp(20f), Theme.dp(3f), Theme.dp(20f), Theme.dp(7f)) })
            row.setDebounceClickListener {
                this@SelectPlanDialog.close()
                this@SelectPlanDialog.fragment.pushFragment(PublicPlanViewFragment.getInstance(plan))
            }
        }
    }

    fun show(listener: SelectPlanListener) {
        super.show()
        try {
            this.listener = listener
            this.bottomDialog?.behavior?.state = BottomSheetBehavior.STATE_EXPANDED
            this.bottomDialog?.show()
            this.bottomDialog?.window?.navigationBarColor = Theme.getColor(Theme.key_whiteBackground)
        } catch (e: Exception) {
            e.printStackTrace()
        }
    }

    fun close() {
        this.bottomDialog?.dismiss()
        this.bag.clear()
    }
}