package nexus.billing.ui.profile

import android.graphics.PorterDuff
import android.os.Bundle
import android.transition.TransitionInflater
import android.util.TypedValue
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.LinearLayout
import android.widget.ScrollView
import android.widget.TextView
import androidx.appcompat.app.AlertDialog
import androidx.fragment.app.viewModels
import com.google.android.material.appbar.MaterialToolbar
import nexus.appui.Theme
import nexus.appui.Theme.dp
import nexus.appui.ui.BaseFragment
import nexus.appui.utils.Resource
import nexus.appui.utils.ViewUtils
import nexus.appui.utils.setDebounceClickListener
import nexus.billing.R
import nexus.billing.methods.common.Route
import nexus.billing.methods.profile.ProfileService
import nexus.billing.methods.service.Service
import nexus.billing.models.PlanObject
import nexus.billing.models.planDesc
import nexus.billing.models.serviceDesc
import nexus.billing.ui.route.PlanRoutesFragment
import nexus.billing.ui.rows.RowPlanPackage
import nexus.core.util.Currency

class ShowInfoFragment: BaseFragment() {

    companion object {
        const val TAG = "ServiceInfoFragment"
        fun getInstance(planObject: PlanObject, serviceInfo: ProfileService? = null, routes: Array<Route> = emptyArray()): ShowInfoFragment {
            val fragment = ShowInfoFragment()
            fragment.planObject = planObject
            fragment.serviceInfo = serviceInfo
            fragment.routes = routes
            fragment.baseTag = TAG
            val args = Bundle()
            fragment.arguments = args
            return fragment
        }
    }
    private var planObject: PlanObject? = null
    private var serviceInfo: ProfileService? = null
    private val viewModel: AddServiceViewModel by viewModels()
    private var routes: Array<Route> = emptyArray()

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        val inflater = TransitionInflater.from(requireContext())
        enterTransition = inflater.inflateTransition(R.transition.slide_right)
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View {
        val context = requireContext()
        val rootView = LinearLayout(context)
        rootView.fitsSystemWindows = true
        rootView.isClickable = true
        rootView.orientation = LinearLayout.VERTICAL
        rootView.setBackgroundColor(theme.getColor(Theme.key_whiteBackground))

        this.toolbar = MaterialToolbar(context)
        rootView.addView(this.toolbar, ViewUtils.linearMatchWrap())
        this.toolbar.setBackgroundColor(theme.getColor(Theme.key_actionBar))
        this.toolbar.setTitleTextColor(theme.getColor(Theme.key_actionBarText))
        this.toolbar.setTitleTextAppearance(context, nexus.appui.R.style.BigTitleToolbar)
        this.toolbar.elevation = dp(5f).toFloat()
        this.toolbar.setNavigationIcon(R.drawable.ic_arrow_back)
        this.toolbar.navigationIcon?.setColorFilter(theme.getColor(Theme.key_actionBarText), PorterDuff.Mode.SRC_IN)
        this.toolbar.setNavigationOnClickListener {
            requireActivity().supportFragmentManager.popBackStack()
        }

        val scroll = ScrollView(requireContext())
        rootView.addView(scroll, ViewUtils.linearMatchWrap())

        val scrollContent = LinearLayout(requireContext())
        scroll.addView(scrollContent, ViewUtils.scrollMatchMatch())
        scrollContent.orientation = LinearLayout.VERTICAL

        val longText = this.serviceInfo?.let { service ->
            val packageItem = RowPlanPackage(requireContext())
            val currency = planObject?.currency ?: Currency.USD
            packageItem.showData(Pair(service, this.planObject?.serviceDesc(service.serviceID)), currency, service.type == Service.Types.`package`, true)
            packageItem.showRoutes(this.routes) {
                pushFragment(PlanRoutesFragment.getInstance(this.routes, currency, this.planObject?.serviceDesc(service.id)))
            }
            scrollContent.addView(packageItem, ViewUtils.linearMatchWrap().apply { setMargins(dp(20f), dp(20f), dp(20f), dp(5f)) })

            this.setTitle(this.planObject?.serviceDesc(service.id)?.title ?: getString(R.string.service))
            packageItem.activeBtn.setDebounceClickListener {
                this.planObject?.id?.let { id ->
                    this.serviceInfo?.id?.let { serviceId ->
                        this.activeService(id, serviceId)
                    }
                }
            }
            packageItem.removeBtn.setDebounceClickListener {
                this.planObject?.id?.let { id ->
                    this.serviceInfo?.id?.let { serviceId ->
                        this.removeService(id, serviceId)
                    }
                }
            }
            this.planObject?.serviceDesc(service.id)?.longText ?: this.planObject?.serviceDesc(service.id)?.shortText
        } ?: run {
            this.setTitle(this.planObject?.planDesc?.title ?: getString(R.string.service))
            this.planObject?.planDesc?.longText ?: this.planObject?.planDesc?.shortText
        }

        val descTv = TextView(context)
        scrollContent.addView(descTv, ViewUtils.linearMatchWrap())
        descTv.text = longText
        descTv.setTextColor(theme.getColor(Theme.key_rowTextBlack))
        descTv.typeface = Theme.font
        descTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.secondaryText)
        descTv.setPadding(dp(20f), dp(20f), dp(20f), dp(10f))
        return rootView
    }

    private fun activeService(profilePlanId: String, serviceId: String) {
        val b = AlertDialog.Builder(requireContext(), Theme.dialogTheme)
        b.setMessage(getString(R.string.want_activate_service))
        b.setNegativeButton(nexus.appui.R.string.cancel) { dialog, _ -> dialog.cancel() }
        b.setPositiveButton(nexus.appui.R.string.ok) { dialog, _ ->
            this.viewModel.activateService(profilePlanId, serviceId).observe(viewLifecycleOwner) { result ->
                when(result.status) {
                    Resource.Status.LOADING -> {
                        this.progressDialog.show()
                    }
                    Resource.Status.SUCCESS -> {
                        this.progressDialog.dismiss()
                        requireActivity().supportFragmentManager.popBackStack()
                    }
                    Resource.Status.ERROR -> {
                        this.progressDialog.dismiss()
                        ViewUtils.showErrorDialog(requireActivity(), null, getString(R.string.request_error_repeat))
                    }
                }
            }
        }
        b.create().show()
    }

    private fun removeService(profilePlanId: String, serviceId: String) {
        val b = AlertDialog.Builder(requireContext(), Theme.dialogTheme)
        b.setMessage(getString(R.string.want_delete_service))
        b.setNegativeButton(nexus.appui.R.string.cancel) { dialog, _ -> dialog.cancel() }
        b.setPositiveButton(nexus.appui.R.string.ok) { dialog, _ ->
            this.viewModel.removeService(profilePlanId, serviceId).observe(viewLifecycleOwner) { result ->
                when(result.status) {
                    Resource.Status.LOADING -> {
                        this.progressDialog.show()
                    }
                    Resource.Status.SUCCESS -> {
                        this.progressDialog.dismiss()
                        requireActivity().supportFragmentManager.popBackStack()
                    }
                    Resource.Status.ERROR -> {
                        this.progressDialog.dismiss()
                        ViewUtils.showErrorDialog(requireActivity(), null, getString(R.string.request_error_repeat))
                    }
                }
            }
        }
        b.create().show()
    }
}