package nexus.billing.ui.route

import android.animation.LayoutTransition
import android.content.res.ColorStateList
import android.graphics.PorterDuff
import android.os.Bundle
import android.text.Editable
import android.text.TextUtils
import android.text.TextWatcher
import android.transition.TransitionInflater
import android.util.TypedValue
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.EditText
import android.widget.FrameLayout
import android.widget.LinearLayout
import android.widget.TextView
import androidx.core.view.setPadding
import androidx.core.widget.TextViewCompat
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.google.android.material.appbar.MaterialToolbar
import io.reactivex.rxjava3.android.schedulers.AndroidSchedulers
import io.reactivex.rxjava3.core.Observable
import io.reactivex.rxjava3.schedulers.Schedulers
import nexus.appui.Theme
import nexus.appui.Theme.dp
import nexus.appui.recycler.SpaceItemDecoration
import nexus.appui.ui.BaseFragment
import nexus.appui.utils.ViewUtils
import nexus.billing.R
import nexus.billing.methods.common.Route
import nexus.billing.models.DescriptionObject
import nexus.billing.ui.rows.RowRoute
import nexus.core.util.Currency
import java.util.concurrent.TimeUnit

class PlanRoutesFragment: BaseFragment() {

    companion object {
        const val TAG = "PlanRoutesFragment"
        fun getInstance(routes: Array<Route>, currency: Currency, desc: DescriptionObject?): PlanRoutesFragment {
            val fragment = PlanRoutesFragment()
            fragment.routes = routes
            fragment.currency = currency
            fragment.desc = desc
            fragment.baseTag = TAG
            val args = Bundle()
            fragment.arguments = args
            return fragment
        }
    }
    private var routes: Array<Route> = emptyArray()
    private lateinit var adapter: RouteAdapter
    private var desc: DescriptionObject? = null
    private var currency: Currency = Currency.USD
    private lateinit var rv: RecyclerView

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        val inflater = TransitionInflater.from(requireContext())
        enterTransition = inflater.inflateTransition(R.transition.slide_right)
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View {
        val context = requireContext()
        val rootView = FrameLayout(context)
        rootView.fitsSystemWindows = true
        rootView.isClickable = true
        rootView.setBackgroundColor(theme.getColor(Theme.key_mainBackground))

        val contentVertical = LinearLayout(context)
        contentVertical.orientation = LinearLayout.VERTICAL
        contentVertical.layoutTransition = LayoutTransition()
        rootView.addView(contentVertical, ViewUtils.frameMatchMatch())

        this.toolbar = MaterialToolbar(context)
        contentVertical.addView(this.toolbar, ViewUtils.frameMatchWrap())
        this.toolbar.setBackgroundColor(theme.getColor(Theme.key_actionBar))
        this.toolbar.setTitleTextColor(theme.getColor(Theme.key_actionBarText))
        this.toolbar.setTitleTextAppearance(context, nexus.appui.R.style.BigTitleToolbar)
        this.toolbar.elevation = dp(5f).toFloat()
        this.setTitle(this.desc?.title ?: getString(R.string.service))
        this.toolbar.setNavigationIcon(R.drawable.ic_arrow_back)
        this.toolbar.navigationIcon?.setColorFilter(theme.getColor(Theme.key_actionBarText), PorterDuff.Mode.SRC_IN)
        this.toolbar.setNavigationOnClickListener {
            requireActivity().supportFragmentManager.popBackStack()
        }

        this.desc?.shortText?.let { shortText ->
            val descTv = TextView(context)
            contentVertical.addView(descTv, ViewUtils.linearMatchWrap())
            descTv.text = shortText
            descTv.setTextColor(theme.getColor(Theme.key_greyColor))
            descTv.typeface = Theme.font
            descTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.secondaryText)
            descTv.setPadding(dp(20f), dp(10f), dp(20f), 0)
        }

        val edtSearch = EditText(context)
        edtSearch.hint = getString(R.string.route_search)
        edtSearch.isSingleLine = true
        edtSearch.setCompoundDrawablesWithIntrinsicBounds(R.drawable.ic_search, 0, 0, 0)
        TextViewCompat.setCompoundDrawableTintList(edtSearch, ColorStateList.valueOf(theme.getColor(Theme.key_greyColor)))
        edtSearch.compoundDrawablePadding = dp(10f)
        edtSearch.elevation = dp(5f).toFloat()
        edtSearch.setHintTextColor(theme.getColor(Theme.key_greyColor))
        edtSearch.setTextColor(theme.getColor(Theme.key_rowTextBlack))
        edtSearch.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.normalText)
        edtSearch.setPadding(dp(10f))
        edtSearch.background = ViewUtils.getRect(theme.getColor(Theme.key_blockView), dp(10f))
        val eLp = ViewUtils.linearMatchWrap()
        eLp.setMargins(dp(20f), dp(15f), dp(20f), dp(20f))
        contentVertical.addView(edtSearch, eLp)
        Observable.create { subscriber ->
            edtSearch.addTextChangedListener(object : TextWatcher {
                override fun beforeTextChanged(s: CharSequence?, start: Int, count: Int, after: Int) {}
                override fun onTextChanged(s: CharSequence, start: Int, before: Int, count: Int) {
                    subscriber.onNext(s.toString())
                }
                override fun afterTextChanged(s: Editable?) {}
            })
        }.debounce(250, TimeUnit.MILLISECONDS)
            .distinctUntilChanged()
            .subscribeOn(Schedulers.io())
            .observeOn(AndroidSchedulers.mainThread())
            .subscribe { text ->
                val newRoutes = if (text.trim().isEmpty()) {
                    this.routes
                } else {
                    this.routes.filter { it.name.contains(text.trim(), true) }.toTypedArray()
                }
                this.adapter.routes = newRoutes
                this.adapter.notifyDataSetChanged()
            }

        val line = LinearLayout(context)
        contentVertical.addView(line, ViewUtils.linearMatchWrap())
        line.orientation = LinearLayout.HORIZONTAL
        line.setPadding(dp(25f), 0, dp(25f), dp(5f))
        val countryTv = TextView(context)
        line.addView(countryTv, ViewUtils.linearParams(0, LinearLayout.LayoutParams.WRAP_CONTENT, 1f))
        countryTv.text = getString(R.string.country_region)
        countryTv.setTextColor(theme.getColor(Theme.key_greyColor))
        countryTv.typeface = Theme.font
        countryTv.isSingleLine = true
        countryTv.ellipsize = TextUtils.TruncateAt.END
        countryTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.secondaryText)
        val priceTv = TextView(context)
        line.addView(priceTv, ViewUtils.linearParams(0, LinearLayout.LayoutParams.WRAP_CONTENT, 1f))
        priceTv.text = "${getString(R.string.rate_per_min)} (${ this.currency.name })"
        priceTv.setTextColor(theme.getColor(Theme.key_greyColor))
        priceTv.typeface = Theme.font
        priceTv.isSingleLine = true
        priceTv.gravity = Gravity.END
        priceTv.ellipsize = TextUtils.TruncateAt.END
        priceTv.setTextSize(TypedValue.COMPLEX_UNIT_SP, Theme.secondaryText)

        this.rv = RecyclerView(context)
        val rvLp = ViewUtils.linearMatchWrap()
        rvLp.setMargins(dp(20f), 0, dp(20f), 0)
        contentVertical.addView(rv, rvLp)
        this.rv.background = ViewUtils.getRect(theme.getColor(Theme.key_blockView), dp(10f))
        this.rv.elevation = dp(5f).toFloat()
        this.rv.layoutManager = LinearLayoutManager(context)
        this.rv.addItemDecoration(SpaceItemDecoration(0))
        this.adapter = RouteAdapter(this.routes)
        this.rv.adapter = this.adapter

        return rootView
    }

    private inner class RouteAdapter(var routes: Array<Route>) : RecyclerView.Adapter<RouteAdapter.RouteHolder>() {
        val localCountries = ViewUtils.getAllCountries(requireContext())
        override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): RouteHolder {
            return RouteHolder(RowRoute(parent.context))
        }

        override fun onBindViewHolder(holder: RouteHolder, position: Int) {
            val route = routes[position]
            val country = this.localCountries.find { it.name.trim() == route.name.trim() }
            val item = holder.itemView as RowRoute
            item.showData(route, true, country)
        }

        override fun getItemCount(): Int {
            return routes.size
        }

        inner class RouteHolder(itemView: View) : RecyclerView.ViewHolder(itemView)
    }
}